﻿

#include<Windows.h>
#include<stdio.h>


void serial_write(const char* data, int len); 
int serial_read(char* buff, int wait_ms);
int receive_char(COMSTAT comstat, char * buff); 
extern void detect_baudrate(void); 
extern bool check_boot_mode(void); 
extern void enter_boot_mode(void); 
extern bool enter_dfu_mode(void);
extern bool ymodem_upgrade(void); 
/**********************************************************************/

#define COM_PORT   L"\\\\.\\COM35"
HANDLE hComm;

int main()
{
	BOOL   Status;
	    
	hComm = CreateFile(COM_PORT,            //port name 
        GENERIC_READ | GENERIC_WRITE, //Read/Write   				 
		0,            // No Sharing                               
		NULL,         // No Security                              
		OPEN_EXISTING,// Open existing port only                     
		0,            // Non Overlapped I/O                           
		NULL);        // Null for Comm Devices

	if (hComm == INVALID_HANDLE_VALUE)
		printf("Error in opening Serial port");
	else
		printf("\nopening serial port successful\n");


	/*------------------------------- Setting the Parameters for the SerialPort ------------------------------*/

	DCB dcbSerialParams = { 0 };                        // Initializing DCB structure
	dcbSerialParams.DCBlength = sizeof(dcbSerialParams);

	Status = GetCommState(hComm, &dcbSerialParams);     //retreives  the current settings

	if (Status == FALSE)
		printf("\n   Error! in GetCommState()");

	dcbSerialParams.BaudRate = CBR_115200;      // Setting BaudRate = 9600
	dcbSerialParams.ByteSize = 8;             // Setting ByteSize = 8
	dcbSerialParams.StopBits = ONESTOPBIT;    // Setting StopBits = 1
	dcbSerialParams.Parity = NOPARITY;      // Setting Parity = None 

	Status = SetCommState(hComm, &dcbSerialParams);  //Configuring the port according to settings in DCB 

	if (Status == FALSE)
	{
		printf("\n   Error! in Setting DCB Structure");
	}
	else
	{
		printf("\n   Setting DCB Structure Successfull\n");
		printf("\n       Baudrate = %d", dcbSerialParams.BaudRate);
		printf("\n       ByteSize = %d", dcbSerialParams.ByteSize);
		printf("\n       StopBits = %d", dcbSerialParams.StopBits);
		printf("\n       Parity   = %d\n", dcbSerialParams.Parity);
	}
	/*------------------------------------ Setting Timeouts --------------------------------------------------*/

	COMMTIMEOUTS timeouts = { 0 };

	timeouts.ReadIntervalTimeout = 50;
	timeouts.ReadTotalTimeoutConstant = 50;
	timeouts.ReadTotalTimeoutMultiplier = 10;
	timeouts.WriteTotalTimeoutConstant = 50;
	timeouts.WriteTotalTimeoutMultiplier = 10;

	if (SetCommTimeouts(hComm, &timeouts) == FALSE)
		printf("\n   Error! in Setting Time Outs\n");

	/*------------------------------------ Setting WaitComm() Event   ----------------------------------------*/
	Status = SetCommMask(hComm, EV_RXCHAR); //Configure Windows to Monitor the serial device for Character Reception

	if (Status == FALSE)
		printf("    Error! in Setting CommMask\n");

    //detect_baudrate(); 
	bool boot_mode= check_boot_mode(); 
	while (!boot_mode)
	{
		enter_boot_mode();
		boot_mode = check_boot_mode();
	}

    enter_dfu_mode(); 
	ymodem_upgrade(); 
    
	CloseHandle(hComm);//Closing the Serial Port

	return 0;
}
//=============================================================
void serial_write(const char *data, int len)
{
    BOOL   Status;
	DWORD  dNoOfBytesWritten = 0;          // No of bytes written to the port

	Status = WriteFile(hComm,               // Handle to the Serialport
		 data,              // Data to be written to the port 
		(DWORD)len,         // No of bytes to write into the port
		&dNoOfBytesWritten, // No of bytes written to the port
		NULL);

	if (Status == FALSE)
		printf("   Error %d in Writing to Serial Port\n", GetLastError());

    /*
    else
		for (int i = 0; i < len; )
		{
			printf("%02X ", (unsigned char)data[i]);
			if (++i % 16==0)	printf("\n");
		}
	printf("\n"); 
    */
}

/*------------------------------------ Setting Receive Mask ----------------------------------------------*/
int serial_read(char *buf, int wait_ms)
{
    BOOL   Status;
	DWORD dwEventMask;                     // Event mask to trigger
	char  TempChar;                        // Temperory Character
	DWORD NoBytesRead;                     // Bytes read by ReadFile()
	int len = 0;

	//memset(buf, 0, 64); 
	Status = WaitCommEvent(hComm, &dwEventMask, NULL); 

    Sleep(wait_ms); 
    
	/* will Wait here till a Character is received */

	if (Status == FALSE){
		printf("\n    Error! in Setting WaitCommEvent()");
	}
	else 
	{
        DWORD dwError = 0;
        COMSTAT comstat;

        if (ClearCommError(hComm, &dwError, &comstat) && dwError > 0){
            PurgeComm(hComm, PURGE_RXABORT);
        } 
        if (comstat.cbInQue){
            len = receive_char(comstat, buf);
        }        
	}
    buf[len]=0; 
    //printf("%s", buf);
    return len; 
}


int receive_char(COMSTAT comstat, char *buffs)
{
	bool bRead = true;
	bool bResult = true;
	DWORD dwError = 0;
	DWORD BytesRead = 0;
	unsigned char RxChar;
    int i=0; 
    
	for (;;)
	{
		bResult = ClearCommError(hComm, &dwError, &comstat);

		if (comstat.cbInQue == 0)
			break;

		if (bRead)
		{
			bResult = ReadFile(hComm, &RxChar, 1, &BytesRead, NULL);
			if (!bResult)
			{
				dwError = GetLastError();
				switch (dwError)
				{
                    case ERROR_IO_PENDING:
                        bRead = FALSE;
                        break;
                    default:
                        printf("ReadFile() error \n");
                        break;
				}
			}
			else
				bRead = true;
		}

		if (!bRead)
		{
			bRead = true;
			bResult = GetOverlappedResult(hComm, NULL, &BytesRead, true);
			if (!bResult)
				printf("GetOverlappedResults() in ReadFile() \n");
		}

		buffs[i++] = RxChar;
	}
	return i; 
}
